@@ -1,3 +1,4 @@
+import copy
 import json
 from pathlib import Path
 from threading import Thread
@@ -12,12 +13,14 @@ from .utils import (
     _get_archive_checksum,
     _get_or_create_resource_archive,
     generate_random_name,
+    make_name_by_path,
 )
 from ..interfaces import (
     EvaluateSubTask,
     EvaluateTask,
     Project,
     ProjectVisibility,
+    QuotaGroup,
     TrainTask,
 )
 from ..interfaces.resource import (
@@ -35,6 +38,7 @@ from ..utils import logger
 from ..utils.errors import (
     AnyLearnException,
     AnyLearnMissingParamException,
+    AnylearnRequiredLocalCommitException,
 )
 
 
@@ -53,7 +57,7 @@ def _get_or_create_dataset(id: Optional[str]=None,
                 "['dir_path', 'archive_path'] "
                 "is specified."
             ))
-        name = f"DSET_{generate_random_name()}"
+        name = make_name_by_path(dir_path or archive_path)
         archive_path = _get_or_create_resource_archive(
             name=name,
             dir_path=dir_path,
@@ -99,7 +103,7 @@ def _get_or_create_model(id: Optional[str]=None,
                 "Parameter 'algorithm' should be specified "
                 "when using local models."
             )
-        name = f"MODE_{generate_random_name()}"
+        name = make_name_by_path(dir_path or archive_path)
         archive_path = _get_or_create_resource_archive(
             name=name,
             dir_path=dir_path,
@@ -198,26 +202,47 @@ def _get_or_create_project(project_id: Optional[str]=None,
         return project
 
 
+def _format_resource_request(resource_request: List[Dict[str, Dict[str, int]]]):
+    if not resource_request:
+        return None
+    # Resource group by ID or name
+    for i, req in enumerate(copy.deepcopy(resource_request)):
+        for key, val in req.items():
+            if key in ['default', 'besteffort'] or key.startswith('QGRP'):
+                # Already ID
+                continue
+            try:
+                qid = QuotaGroup(name=key, load_detail=True).id
+            except AnyLearnException:
+                raise AnyLearnException(f"Failed to find QuotaGroup: {key}")
+            resource_request[i][qid] = val
+            del resource_request[i][key]
+    return resource_request
+
+
 def _create_train_task(name: str,
                        algorithm: Algorithm,
-                       dataset: Dataset, # deprecated 0.10.9 - remove 0.11.0
                        project: Project,
-                       dataset_hyperparam_name: str, # deprecated 0.10.9 - remove 0.11.0
                        hyperparams: dict,
-                       datasets: Optional[Dict[str, Dataset]]=None,
-                       gpu_num: int=0, # deprecated 0.11.0 - remove soon
-                       gpu_mem: int=0, # deprecated 0.11.0 - remove soon
-                       resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None):
-    files = [v.id for v in datasets.values()]
+                       mounts: Optional[Dict[str, Resource]]=None,
+                       algorithm_git_ref: Optional[str]=None,
+                       resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None,
+                       description: Optional[str]=None):
+    files = [v.id for v in mounts.values()]
     train_params = dict(
-        {k: f"${v.id}" for k, v in datasets.items()},
+        {k: f"${v.id}" for k, v in mounts.items()},
         **hyperparams,
     )
-    train_task = TrainTask(name=name, project_id=project.id,
-                           algorithm_id=algorithm.id,
-                           files=files,
-                           train_params=json.dumps(train_params),
-                           resource_request=resource_request)
+    train_task = TrainTask(
+        name=name,
+        project_id=project.id,
+        algorithm_id=algorithm.id,
+        algorithm_git_ref=algorithm_git_ref,
+        files=files,
+        train_params=json.dumps(train_params),
+        resource_request=resource_request,
+        description=description,
+    )
     train_task.save()
     train_task.get_detail()
     return train_task
@@ -250,10 +275,17 @@ def _create_eval_task(name: str,
 def quick_train(algorithm_id: Optional[str]=None,
                 algorithm_name: Optional[str]=None,
                 algorithm_dir: Optional[Union[str, Path]]=None,
-                algorithm_archive: Optional[str]=None,
+                algorithm_archive: Optional[str]=None, # deprecated in 0.11.11, remove in 0.12.3
+                algorithm_force_update: bool=False,
+                algorithm_git_ref: Optional[str]=None,
+                dataset_hyperparam_name: str="dataset",
                 dataset_id: Optional[str]=None,
                 dataset_dir: Optional[Union[str, Path]]=None,
                 dataset_archive: Optional[str]=None,
+                model_hyperparam_name: str="model",
+                model_id: Optional[str]=None,
+                pretrain_hyperparam_name: str="pretrain",
+                pretrain_task_id: Optional[str]=None,
                 project_id: Optional[str]=None,
                 project_name: Optional[str]=None,
                 entrypoint: Optional[str]=None,
@@ -261,11 +293,9 @@ def quick_train(algorithm_id: Optional[str]=None,
                 mirror_name: Optional[str]="QUICKSTART",
                 resource_uploader: Optional[ResourceUploader]=None,
                 resource_polling: Union[float, int]=5,
-                dataset_hyperparam_name: str="dataset",
                 hyperparams: dict={},
-                gpu_num: int=0, # deprecated 0.11.0 - remove soon
-                gpu_mem: int=0, # deprecated 0.11.0 - remove soon
-                resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None):
+                resource_request: Optional[List[Dict[str, Dict[str, int]]]]=None,
+                task_description: Optional[str]=None):
     """
     本地算法快速训练接口。
 
@@ -309,12 +339,42 @@ def quick_train(algorithm_id: Optional[str]=None,
         本地算法目录路径。
     algorithm_archive : :obj:`str`, optional
         本地算法压缩包路径。
+
+        .. deprecated:: 0.12.0
+
+            Not supported anymore.
+
+    algorithm_git_ref : :obj:`str`, optional
+        算法Gitea代码仓库的版本号（可以是commit号、分支名、tag名）。
+        使用本地算法时，如未提供此参数，则取本地算法当前分支名。
+    algorithm_force_update : :obj:`bool`, optional
+        在同步算法的过程中是否强制更新算法，如为True，Anylearn会对未提交的本地代码变更进行自动提交。默认为False。
+    dataset_hyperparam_name : :obj:`str`, optional
+        启动训练时，数据集路径作为启动命令参数传入算法的参数名。
+        需指定长参数名，如 :obj:`--data` ，并省略 :obj:`--` 部分传入。
+        数据集路径由Anylearn后端引擎管理。
+        默认为 :obj:`dataset` 。
     dataset_id : :obj:`str`, optional
         已在Anylearn远程注册的数据集ID。
     dataset_dir : :obj:`str`, optional
         本地数据集目录路径。
     dataset_archive : :obj:`str`, optional
         本地数据集压缩包路径。
+    model_hyperparam_name : :obj:`str`, optional
+        启动训练时，模型路径作为启动命令参数传入算法的参数名。
+        需指定长参数名，如 :obj:`--model` ，并省略 :obj:`--` 部分传入。
+        模型路径由Anylearn后端引擎管理。
+        默认为 :obj:`model` 。
+    model_id : :obj:`str`, optional
+        已在Anylearn远程注册/转存的模型ID。
+    pretrain_hyperparam_name: :obj:`str`, optional
+        启动训练时，前置训练结果（间接抽象为“预训练”，即"pretrain"）路径作为启动命令参数传入算法的参数名。
+        需指定长参数名，如 :obj:`--pretrain` ，并省略 :obj:`--` 部分传入。
+        预训练结果路径由Anylearn后端引擎管理。
+        默认为 :obj:`pretrain` 。
+    pretrain_task_id: :obj:`str`, optional
+        在Anylearn进行过的训练的ID，一般为前缀TRAI的32位字符串。
+        Anylearn会对指定的训练进行结果抽取并挂载到新一次的训练中。
     project_id : :obj:`str`, optional
         已在Anylearn远程创建的训练项目ID。
     entrypoint : :obj:`str`, optional
@@ -327,42 +387,17 @@ def quick_train(algorithm_id: Optional[str]=None,
     resource_polling : :obj:`float|int`, optional
         资源上传中轮询资源状态的时间间隔（单位：秒）。
         默认为5秒。
-    dataset_hyperparam_name : :obj:`str`, optional
-        启动训练时，数据集路径作为启动命令参数传入算法的参数名。
-        需指定长参数名，如 :obj:`--data` ，并省略 :obj:`--` 部分传入。
-        数据集路径由Anylearn后端引擎管理。
-        默认为 :obj:`dataset` 。
     hyperparams : :obj:`dict`, optional
         训练超参数字典。
         超参数将作为训练启动命令的参数传入算法。
         超参数字典中的键应为长参数名，如 :obj:`--param` ，并省略 :obj:`--` 部分传入。
         如需要标识类参数（flag），可将参数的值设为空字符串，如 :obj:`{'my-flag': ''}` ，等价于 :obj:`--my-flag` 传入训练命令。
         默认为空字典。
-    gpu_num : :obj:`int`, optional
-        向Anylearn后端引擎请求的GPU数量，
-        与 :obj:`gpu_mem` 参数互斥，
-        同时设置时 :obj:`gpu_mem` 优先。
-        默认为0。
-
-        .. deprecated:: 0.11.0
-
-            Use :obj:`resources_request` instead
-            to specify QuotaGroup and resource request.
-
-    gpu_mem : :obj:`int`, optional
-        向Anylearn后端引擎请求的显存大小，
-        与 :obj:`gpu_num` 参数互斥，
-        同时设置时 :obj:`gpu_mem` 优先。
-        默认为0。
-
-        .. deprecated:: 0.11.0
-
-            Use :obj:`resources_request` instead
-            to specify QuotaGroup and resource request.
-
     resource_request : :obj:`List[Dict[str, Dict[str, int]]]`, optional
         训练所需计算资源的请求。
         如未填，则使用Anylearn后端的:obj:`default`资源组中的默认资源套餐。
+    task_description : :obj:`str`, optional
+        训练任务详细描述。
 
     Returns
     -------
@@ -375,17 +410,39 @@ def quick_train(algorithm_id: Optional[str]=None,
     Project
         创建的训练项目对象
     """
+    # Resource request
+    resource_request = _format_resource_request(resource_request)
+
+    # Remote odel
+    model = Model(id=model_id, load_detail=True) if model_id else None
+
+    # Remote pretrain task results
+    pretrain_task = (
+        TrainTask(id=pretrain_task_id, load_detail=True)
+        if pretrain_task_id
+        else None
+    )
+
     # Algorithm
-    algo = sync_algorithm(
-        id=algorithm_id,
-        name=algorithm_name,
-        dir_path=algorithm_dir,
-        archive_path=algorithm_archive,
-        entrypoint_training=entrypoint,
-        output_training=output,
-        mirror_name=mirror_name,
-        uploader=resource_uploader,
-        polling=resource_polling,
+    try:
+        algo, current_sha = sync_algorithm(
+            id=algorithm_id,
+            name=algorithm_name,
+            dir_path=algorithm_dir,
+            entrypoint_training=entrypoint,
+            output_training=output,
+            mirror_name=mirror_name,
+            uploader=resource_uploader,
+            polling=resource_polling,
+            force=algorithm_force_update,
+        )
+    except AnylearnRequiredLocalCommitException:
+        # Notify possible usage of algorithm_force_update=True
+        raise AnylearnRequiredLocalCommitException(
+            f"Algorithm dir {algorithm_dir} is not clean. "
+            "Please commit your changes or "
+            "specify `algorithm_force_update=True` "
+            "to let Anylearn make an auto-commit."
     )
 
     # Dataset
@@ -402,9 +459,13 @@ def quick_train(algorithm_id: Optional[str]=None,
                         polling=resource_polling)
         DB().create_local_dataset(id=dset.id, checksum=dataset_checksum)
 
-    datasets = {}
+    mounts = {}
     if dset and dataset_hyperparam_name:
-        datasets[dataset_hyperparam_name] = dset
+        mounts[dataset_hyperparam_name] = dset
+    if model and model_hyperparam_name:
+        mounts[model_hyperparam_name] = model
+    if pretrain_task and pretrain_hyperparam_name:
+        mounts[pretrain_hyperparam_name] = pretrain_task.get_results_file()
 
     # Project
     if project_id or project_name:
@@ -422,12 +483,12 @@ def quick_train(algorithm_id: Optional[str]=None,
     train_task = _create_train_task(
         name=train_task_name,
         algorithm=algo,
-        dataset=dset, # deprecated 0.10.9 - remove 0.11.0
+        algorithm_git_ref=algorithm_git_ref or current_sha,
         project=project,
-        dataset_hyperparam_name=dataset_hyperparam_name, # deprecated 0.10.9 - remove 0.11.0
         hyperparams=hyperparams,
-        datasets=datasets,
+        mounts=mounts,
         resource_request=resource_request,
+        description=task_description,
     )
 
     DB().create_or_update_train_task(train_task=train_task)
